/*
 * Copyright (C) 1996 - 2000 BULL
 * Copyright (C) 1996 - 2000 INRIA
 *
 * The contents of this file are subject to the Joram Public License,
 * as defined by the file JORAM_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is Joram, including the java packages fr.dyade.aaa.agent,
 * fr.dyade.aaa.util, fr.dyade.aaa.ip, fr.dyade.aaa.mom, and fr.dyade.aaa.joram,
 * released May 24, 2000. 
 * 
 * The Initial Developer of the Original Code is Dyade. The Original Code and
 * portions created by Dyade are Copyright Bull and Copyright INRIA.
 * All Rights Reserved.
 */
package fr.dyade.aaa.agent;

import org.objectweb.util.monolog.api.BasicLevel;

/** 
 * The <code>DriverConnect</code> class is used by <code>ProxyAgent</code>
 * instances for managing their connection steps.
 */
class DriverConnect extends Driver {
  protected ProxyAgent proxy = null;
  protected boolean blockingCnx;
  protected boolean multipleCnx;

  DriverConnect(ProxyAgent proxy, boolean blockingCnx, boolean multipleCnx) {
    this.proxy = proxy;
    this.blockingCnx = blockingCnx;
    this.multipleCnx = multipleCnx;
    this.name = proxy.getName() + ".DriverConnect";
    // Get the proxy logging monitor
    logmon = proxy.logmon;
  }


  public void start() {
    if (! blockingCnx)
      run();
    else
      super.start();
  }

  public void run() {
    try {
      if (logmon.isLoggable(BasicLevel.DEBUG))
        logmon.log(BasicLevel.DEBUG, getName() + ", started");

      do {
        proxy.createDrivers();
      } while (multipleCnx);
    }
    catch (java.net.SocketException exc) {
      logmon.log(BasicLevel.WARN, "connection closed in createDrivers()", exc);
    }
    catch (java.io.EOFException exc) {
      logmon.log(BasicLevel.WARN, "connection closed in createDrivers()", exc);
    }
    catch (Exception exc) { 
      logmon.log(BasicLevel.ERROR, "error in createDrivers()", exc);
    }
  }

  public void close() {}
}
